use rand::prelude::*;
use rand_chacha::ChaCha8Rng;

/// Own RNG abstraction; seedable but not secure (seeded from a single number)
pub struct Rand {
    rng: ChaCha8Rng,
}

impl Rand {
    pub fn new_from_seed(seed: u64) -> Self {
        Rand {
            rng: ChaCha8Rng::seed_from_u64(seed),
        }
    }

    pub fn random_byte(&mut self) -> u8 {
        self.rng.gen_range(0..=255)
    }

    pub fn random_range(&mut self, max: u64) -> u64 {
        self.rng.gen_range(0..max)
    }
}

#[cfg(test)]
mod test {
    use super::Rand;

    #[test]
    fn rand_1() {
        let mut r = Rand::new_from_seed(1);
        let n1 = r.random_byte();
        assert_eq!(n1, 177);
    }

    #[test]
    fn rand_10() {
        let mut r = Rand::new_from_seed(1);
        assert_eq!(r.random_byte(), 177);
        assert_eq!(r.random_byte(), 234);
        for _i in 0..7 {
            let _n = r.random_byte();
        }
        assert_eq!(r.random_byte(), 83);
    }

    #[test]
    fn rand_diff_seed() {
        assert_eq!(Rand::new_from_seed(1).random_byte(), 177);
        assert_eq!(Rand::new_from_seed(2).random_byte(), 197);
        assert_eq!(Rand::new_from_seed(3).random_byte(), 93);
        assert_eq!(Rand::new_from_seed(1).random_byte(), 177);
    }
}
